#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       Strip
#       
#       Copyright 2011 Adrien Plazas <kekun.plazas@laposte.net>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.



import os
import gtk
import archive
import tempfile
import shutil
import threading

class Book(object):
    def __init__(self, path, image_types, archive_types):
        '''Create a book wich store the path to its pages and informations'''
        self._tmp_dir = tempfile.mkdtemp(prefix='strip.', suffix=os.sep)
        self.folder = ''
        self.page = []
        self.page_list = []
        self.length = 0
        self.image_types = image_types
        self.archive_types = archive_types
        
        if os.path.isdir(path):
            self.folder = path
            self.page_list = self.open_image_file(image_types)
        else:
            fileType = path.split(".")[-1]
            for type in self.image_types:
                if type == fileType:
                    self.folder = path[:len(path)-len(os.path.basename(path))]
                    self.page_list = self.open_image_file(os.listdir(self.folder))
            for type in self.archive_types:
                if type == fileType:
                    self.page_list = self.open_archive_file(path)
        self.length = len(self.page_list)
        self.page = self.create_pages(self.folder, self.page_list)
        
    def open_image_file(self, folder_content):
        page_list = []
        for file in folder_content:
            fileType = file.split(".")[-1]
            for type in  self.image_types:
                if type == fileType:
                    page_list.append(file)
        return sorted(page_list)
        
    def open_archive_file(self, path):
        self.extractor =  archive.Extractor()
        self.extractor.setup(path, self._tmp_dir)
        self.folder = self._tmp_dir
        page_list = self.extractor.get_files()
        self.extractor.set_files(page_list)
        self.extractor.extract()
        while 1:
            i = 0
            for file in page_list:  
                if self.extractor.is_ready(file) == False:
                    i+=1
            if i == 0:
                break
        return self.open_image_file(page_list)
    
    def create_pages(self, folder, page_list):
        pages = []
        for file in page_list:
            pages.append(Page(str(self.folder + '/' + file)))
        return pages
    
    def close(self):
        self.thread_delete(self._tmp_dir)
        
    def thread_delete(self, path):
        """Start a threaded removal of the directory tree rooted at <path>.
        This is to avoid long blockings when removing large temporary dirs.
        """
        del_thread = threading.Thread(target=shutil.rmtree, args=(path,))
        del_thread.setDaemon(False)
        del_thread.start()        
    
class Page(object):
    def __init__(self, path):
        self.path = path
    
    def get_image(self):
        image = gtk.Image()
        image.set_from_file(self.path)
        return image
